@@ -1,11 +1,15 @@
 from .alpaca_backtesting import AlpacaBacktesting
-from .alpaca_backtesting import AlpacaBacktesting
 from .alpha_vantage_backtesting import AlphaVantageBacktesting
 from .backtesting_broker import BacktestingBroker
+from .ccxt_backtesting import CcxtBacktesting
+from .interactive_brokers_rest_backtesting import InteractiveBrokersRESTBacktesting
 from .pandas_backtesting import PandasDataBacktesting
 from .polygon_backtesting import PolygonDataBacktesting
 from .thetadata_backtesting import ThetaDataBacktesting
 from .yahoo_backtesting import YahooDataBacktesting
-from .ccxt_backtesting import CcxtBacktesting
-from .interactive_brokers_rest_backtesting import InteractiveBrokersRESTBacktesting
-from .databento_backtesting import DataBentoDataBacktesting
+
+# Import DataBento backtesting - use polars by default if available
+try:
+    from .databento_backtesting_polars import DataBentoDataBacktestingPolars as DataBentoDataBacktesting
+except ImportError:
+    from .databento_backtesting import DataBentoDataBacktesting
\ No newline at end of file
