@@ -1,30 +1,29 @@
-import logging
 import os
+from datetime import datetime, timedelta
+from decimal import Decimal
 from typing import Optional
 
-import pytz
-from datetime import datetime, timedelta
-from decimal import Decimal, ROUND_HALF_EVEN
-
 import pandas as pd
-from alpaca.data.historical import CryptoHistoricalDataClient
-from alpaca.data.historical import StockHistoricalDataClient
+import pytz
+from alpaca.data.historical import CryptoHistoricalDataClient, StockHistoricalDataClient
 from alpaca.data.requests import CryptoBarsRequest, StockBarsRequest
-from alpaca.data.timeframe import TimeFrame, TimeFrameUnit
+from alpaca.data.timeframe import TimeFrame
 
-from lumibot.data_sources import DataSourceBacktesting, AlpacaData
+from lumibot.constants import LUMIBOT_CACHE_FOLDER
+from lumibot.data_sources import AlpacaData, DataSourceBacktesting
 from lumibot.entities import Asset, Bars
-from lumibot import (
-    LUMIBOT_CACHE_FOLDER,
-)
 from lumibot.tools.helpers import (
     date_n_trading_days_from_date,
+    get_decimals,
+    get_timezone_from_datetime,
     get_trading_days,
     get_trading_times,
-    get_timezone_from_datetime,
-    get_decimals,
-    quantize_to_num_decimals
+    quantize_to_num_decimals,
 )
+from lumibot.tools.lumibot_logger import get_logger
+
+logger = get_logger(__name__)
+
 from lumibot.tools.alpaca_helpers import sanitize_base_and_quote_asset
 
 
@@ -148,7 +147,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
             raise ValueError("datetime_start and datetime_end must have the same tzinfo.")
 
         # Get timezone from datetime_start if it has one, otherwise use Lumibot default
-        self._tzinfo = get_timezone_from_datetime(datetime_start)
+        self.tzinfo = get_timezone_from_datetime(datetime_start)
 
         # We want self._data_datetime_start and self._data_datetime_end to be the start and end dates
         # of the data for the entire backtest including the warmup dates.
@@ -159,7 +158,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
             month=datetime_start.month,
             day=datetime_start.day,
         )
-        start_dt = self._tzinfo.localize(start_dt)  # Use localize instead of tzinfo in constructor
+        start_dt = self.tzinfo.localize(start_dt)  # Use localize instead of tzinfo in constructor
 
         # The end should be the last minute of the day.
         end_dt = datetime(
@@ -170,7 +169,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
             minute=59,
             second=59,
         )
-        end_dt = self._tzinfo.localize(end_dt)  # Use localize instead of tzinfo in constructor
+        end_dt = self.tzinfo.localize(end_dt)  # Use localize instead of tzinfo in constructor
 
         if warm_up_trading_days > 0:
             warm_up_start_dt = date_n_trading_days_from_date(
@@ -181,7 +180,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
             # Combine with a default time (midnight)
             warm_up_start_dt = datetime.combine(warm_up_start_dt, datetime.min.time())
             # Make it timezone-aware
-            warm_up_start_dt = self._tzinfo.localize(warm_up_start_dt)
+            warm_up_start_dt = self.tzinfo.localize(warm_up_start_dt)
         else:
             warm_up_start_dt = start_dt
 
@@ -195,7 +194,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
             self.market,
             self._data_datetime_start,
             self._data_datetime_end + timedelta(days=1),  # end_date is exclusive in this function
-            tzinfo=self._tzinfo
+            tzinfo=self.tzinfo
         )
 
         # I think lumibot's got a bug in the strategy_executor when backtesting daily strategies.
@@ -246,7 +245,14 @@ class AlpacaBacktesting(DataSourceBacktesting):
         # as the last price). This approach works for daily and minute bars. For daily bars, this returns the open
         # price, even if now is 9:30 and the daily bar was indexed at 00:00. Thats the only weird thing. But it makes
         # sense. The open of the daily bar for stocks was not at 00:00. It was at 9:30 anyway.
-        price = bars.df.iloc[0].open
+        # Support both pandas and polars-backed Bars without exceptions
+        df_local = bars.df
+        if hasattr(df_local, "iloc"):
+            # pandas: scalar-fast path
+            price = df_local["open"].iat[0]
+        else:
+            # polars
+            price = df_local["open"][0]
         num_decimals = get_decimals(price)
         return quantize_to_num_decimals(price, num_decimals)
 
@@ -259,6 +265,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
             quote: Asset | None = None,
             exchange: str | None = None,
             include_after_hours: bool = True,
+            return_polars: bool = False,
             remove_incomplete_current_bar: Optional[bool] = None,
     ) -> Bars | None:
         """
@@ -367,7 +374,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
         else:
             result_df = df.iloc[max(0, current_index - length + 1): current_index + 1]
 
-        return Bars(result_df, self.SOURCE, asset=asset, quote=quote)
+        return Bars(result_df, self.SOURCE, asset=asset, quote=quote, return_polars=return_polars)
 
     def get_chains(self, asset, quote=None):
         """Mock implementation for getting option chains"""
@@ -420,7 +427,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
             data_datetime_end = self._data_datetime_end
 
         if tzinfo is None:
-            tzinfo = self._tzinfo
+            tzinfo = self.tzinfo
 
         if auto_adjust is None:
             auto_adjust = self._auto_adjust
@@ -494,7 +501,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
         filename = f"{key}.csv"
         filepath = os.path.join(cache_dir, filename)
 
-        logging.info(f"Fetching and caching data for {key}")
+        logger.info(f"Fetching and caching data for {key}")
 
         if base_asset.asset_type == 'crypto':
             client = self._crypto_client
@@ -560,7 +567,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
         # Store in _data_store
         df.set_index('timestamp', inplace=True)
         self._data_store[key] = df
-        logging.info(f"Finished fetching and caching data for {key}")
+        logger.info(f"Finished fetching and caching data for {key}")
         return df
 
     def _load_ohlcv_into_data_store(self, key: str) -> bool:
@@ -595,16 +602,16 @@ class AlpacaBacktesting(DataSourceBacktesting):
             # utc=True ensures proper handling of timezone-aware data
             df['timestamp'] = pd.to_datetime(df['timestamp'], utc=True)
 
-            # Convert timestamps from UTC to the timezone specified in self._tzinfo
-            # For example: if self._tzinfo is 'America/New_York', converts UTC times to NY time
-            df['timestamp'] = df['timestamp'].dt.tz_convert(self._tzinfo)
+            # Convert timestamps from UTC to the timezone specified in self.tzinfo
+            # For example: if self.tzinfo is 'America/New_York', converts UTC times to NY time
+            df['timestamp'] = df['timestamp'].dt.tz_convert(self.tzinfo)
 
             df.set_index('timestamp', inplace=True)
             self._data_store[key] = df
-            logging.info(f"Loaded cached data for key: {key} from cache.")
+            logger.info(f"Loaded cached data for key: {key} from cache.")
             return True
         except Exception as e:
-            logging.error(f"Failed to load cached data for key: {key}. Error: {e}")
+            logger.error(f"Failed to load cached data for key: {key}. Error: {e}")
             return False
 
     def get_historical_prices_between_dates(
@@ -635,7 +642,7 @@ class AlpacaBacktesting(DataSourceBacktesting):
             market = self.market
 
         if tzinfo is None:
-            tzinfo = self._tzinfo
+            tzinfo = self.tzinfo
 
         if data_datetime_start is None:
             data_datetime_start = self._data_datetime_start
