@@ -1,9 +1,5 @@
-import logging
 import traceback
-from collections import OrderedDict
-from datetime import date, timedelta
-from decimal import Decimal
-from typing import Union
+from datetime import datetime, timedelta
 
 import pandas as pd
 
@@ -13,7 +9,8 @@ from lumibot.entities import Asset, Data
 from lumibot.tools import databento_helper
 from lumibot.tools.helpers import to_datetime_aware
 
-logger = logging.getLogger(__name__)
+from lumibot.tools.lumibot_logger import get_logger
+logger = get_logger(__name__)
 
 START_BUFFER = timedelta(days=5)
 
@@ -267,9 +264,9 @@ class DataBentoDataBacktesting(PandasData):
                     df=empty_df,
                     timestep=ts_unit,
                     quote=quote_asset,
-                    # Explicitly set dates to avoid timezone issues
-                    date_start=None,
-                    date_end=None
+                    # Use timezone-aware dates to avoid timezone issues
+                    date_start=LUMIBOT_DEFAULT_PYTZ.localize(datetime(2000, 1, 1)),
+                    date_end=LUMIBOT_DEFAULT_PYTZ.localize(datetime(2000, 1, 1))
                 )
                 self.pandas_data[search_asset] = data_obj
                 return
