@@ -1,8 +1,6 @@
 import os
-import logging
 import time
 from abc import ABC, abstractmethod
-from asyncio.log import logger
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from datetime import datetime, timedelta, timezone
 from decimal import Decimal
@@ -15,6 +13,11 @@ import pandas_market_calendars as mcal
 from dateutil import tz
 from termcolor import colored
 
+from lumibot.tools.lumibot_logger import get_logger
+
+logger = get_logger(__name__)
+
+from lumibot.tools.lumibot_logger import get_logger, get_strategy_logger
 from ..data_sources import DataSource
 from ..entities import Asset, Order, Position, Quote
 from ..trading_builtins import SafeList
@@ -51,22 +54,6 @@ DEFAULT_CLEANUP_CONFIG = {
 class LumibotBrokerAPIError(Exception):
     pass
 
-
-class CustomLoggerAdapter(logging.LoggerAdapter):
-    def process(self, msg, kwargs):
-        # Check if the level is enabled to avoid formatting costs if not necessary
-        if self.logger.isEnabledFor(kwargs.get('level', logging.INFO)):
-            # Lazy formatting of the message
-            return f'[{self.extra["strategy_name"]}] {msg}', kwargs
-        else:
-            return msg, kwargs
-
-    def update_strategy_name(self, new_strategy_name):
-        self.extra['strategy_name'] = new_strategy_name
-        # Pre-format part of the log message that's static or changes infrequently
-        self.formatted_prefix = f'[{new_strategy_name}]'
-
-
 class Broker(ABC):
     # Metainfo
     IS_BACKTESTING_BROKER = False
@@ -120,13 +107,25 @@ class Broker(ABC):
         self._last_cleanup_time = None
 
         # Create an adapter with 'strategy_name' set to the instance's name
-        self.logger = CustomLoggerAdapter(logger, {'strategy_name': "unknown"})
+        self.logger = get_strategy_logger(__name__, "unknown")
 
         # --- Market calendar setting ---
         # StrategyExecutor relies on broker.market to decide whether trading is
         # 24/7 or should follow an exchange calendar.  Derive it from config or
         # env, else default to "NASDAQ" which is compatible with pandas-market-calendars.
         self.market = (config.get("MARKET") if config else None) or os.environ.get("MARKET") or "NASDAQ"
+        # Auto-adjust market for certain known non-equity data sources when still at generic default.
+        try:
+            if self.market in ("NASDAQ", "NYSE", "stock") and self.data_source is not None:
+                ds_name = self.data_source.__class__.__name__
+                if ds_name in ("ProjectXData", "TradovateData"):
+                    self.market = "us_futures"
+                    logger.debug(f"Auto-set broker.market to 'us_futures' for data source {ds_name}")
+                elif ds_name in ("CcxtData", "CcxtBacktestingData", "CCXTData"):
+                    self.market = "24/7"
+                    logger.debug(f"Auto-set broker.market to '24/7' for data source {ds_name}")
+        except Exception as _auto_mkt_exc:
+            logger.debug(f"Auto market inference skipped: {_auto_mkt_exc}")
 
         if self.data_source is None:
             raise ValueError("Broker must have a data source")
@@ -143,6 +142,27 @@ class Broker(ABC):
             if self.stream is not None:
                 self._launch_stream()
 
+        # Trading calendar placeholder; StrategyExecutor will initialize.
+        self._trading_days = None
+
+    # --- Trading calendar initialization ---
+    def initialize_market_calendars(self, trading_days_df):
+        """Initialize broker trading calendar from a DataFrame.
+
+        Sorts by market_close and sets index for fast lookups; stores
+        DataFrame on the broker as `_trading_days`.
+        """
+        if trading_days_df is None:
+            self._trading_days = None
+            return
+        try:
+            df = trading_days_df.sort_values('market_close').copy()
+            df.set_index('market_close', inplace=True)
+            self._trading_days = df
+        except Exception:
+            # If trading_days_df is already indexed/sorted, accept as-is
+            self._trading_days = trading_days_df
+
     def _update_attributes_from_config(self, config):
         value_dict = config
         if not isinstance(config, dict):
@@ -885,9 +905,9 @@ class Broker(ABC):
             # Try to use the limit price if available, otherwise skip processing
             if hasattr(order, 'limit_price') and order.limit_price is not None:
                 price = order.limit_price
-                logging.debug(f"Using limit_price {price} for crypto quote processing since avg_fill_price was None for order {order.identifier}")
+                logger.debug(f"Using limit_price {price} for crypto quote processing since avg_fill_price was None for order {order.identifier}")
             else:
-                logging.debug(f"Skipping crypto quote processing for order {order.identifier} - both avg_fill_price and limit_price are None")
+                logger.debug(f"Skipping crypto quote processing for order {order.identifier} - both avg_fill_price and limit_price are None")
                 return
 
         quote_quantity = Decimal(quantity) * Decimal(price)
@@ -960,6 +980,72 @@ class Broker(ABC):
     def market_open_time(self):
         return self.utc_to_local(self.market_hours(close=False))
 
+    def _is_continuous_market(self, market_name):
+        """
+        Determine if a market trades continuously (24/7 or near-24/7) by checking its trading schedule.
+        
+        This method uses pandas_market_calendars to check actual trading hours and caches results
+        to avoid expensive repeated lookups.
+        
+        Args:
+            market_name (str): Name of the market (e.g., 'NYSE', 'us_futures', '24/7')
+            
+        Returns:
+            bool: True if market trades continuously (>=20 hours per day), False otherwise
+        """
+
+        if not hasattr(self, '_market_type_cache'):
+            self._market_type_cache = {}
+            
+        if market_name in self._market_type_cache:
+            result = self._market_type_cache[market_name]
+            
+            return result
+            
+        try:
+            # Special cases that are definitely continuous
+            if market_name == '24/7':
+                self._market_type_cache[market_name] = True
+                
+                return True
+                
+            # Get market calendar
+            import pandas as pd
+            import pandas_market_calendars as mcal
+            cal = mcal.get_calendar(market_name)
+            
+            # Test with a recent Monday (typical trading day)
+            test_date = pd.Timestamp('2025-01-13', tz='UTC')  # Monday
+            schedule = cal.schedule(start_date=test_date, end_date=test_date)
+            
+            if schedule.empty:
+                # No trading on this date, assume it's not continuous
+                self._market_type_cache[market_name] = False
+                
+                return False
+                
+            # Calculate trading hours duration
+            market_open = schedule.iloc[0, 0]
+            market_close = schedule.iloc[0, 1]
+            duration_hours = (market_close - market_open).total_seconds() / 3600
+            
+            # Consider markets with 20+ hours per day as continuous
+            # This catches futures markets that trade ~22-24 hours
+            is_continuous = duration_hours >= 20.0
+            
+            self._market_type_cache[market_name] = is_continuous
+            
+            return is_continuous
+            
+        except Exception as e:
+            # If we can't determine market type, default to non-continuous for safety
+            # Log the error for debugging
+            
+            if hasattr(self, 'logger'):
+                self.logger.warning(f'Could not determine market type for {market_name}: {e}')
+            self._market_type_cache[market_name] = False
+            return False
+
     def is_market_open(self):
         """Determines if the market is open.
 
@@ -977,13 +1063,44 @@ class Broker(ABC):
         >>> self.is_market_open()
         True
         """
-        open_time = self.utc_to_local(self.market_hours(close=False))
-        close_time = self.utc_to_local(self.market_hours(close=True))
 
-        current_time = datetime.now().astimezone(tz=tz.tzlocal())
-        if self.market == "24/7":
+        # Handle 24/7 markets immediately
+        if self.market == '24/7':
+            
             return True
-        return (current_time >= open_time) and (close_time >= current_time)
+            
+        # Check if this is a continuous market (futures, forex, crypto)
+        if self._is_continuous_market(self.market):
+            
+            return True
+            
+        current_time = datetime.now().astimezone(tz=tz.tzlocal())
+
+        # For ANY market, check both today's and tomorrow's sessions since trading sessions 
+        # can span multiple calendar days (futures: 6pm Thu -> 6pm Fri, forex: Sun 5pm -> Fri 5pm, 
+        # crypto sessions, international markets, etc.)
+        
+        # Check today's session
+        try:
+            open_time_today = self.utc_to_local(self.market_hours(close=False, next=False))
+            close_time_today = self.utc_to_local(self.market_hours(close=True, next=False))
+            
+            if (current_time >= open_time_today) and (close_time_today >= current_time):
+                return True
+        except:
+            pass  # Today might not have a session
+        
+        # Check tomorrow's session (which might have started today)
+        try:
+            open_time_tomorrow = self.utc_to_local(self.market_hours(close=False, next=True))
+            close_time_tomorrow = self.utc_to_local(self.market_hours(close=True, next=True))
+            
+            if (current_time >= open_time_tomorrow) and (close_time_tomorrow >= current_time):
+                return True
+        except:
+            pass  # Tomorrow might not have a session
+        
+        return False
 
     def get_time_to_open(self):
         """Return the remaining time for the market to open in seconds"""
