@@ -10,13 +10,13 @@ import os
 import sys
 
 from .brokers import Alpaca, Ccxt, InteractiveBrokers, InteractiveBrokersREST, Tradier, Tradovate, Schwab, Bitunix, ProjectX
-import logging
 from dotenv import load_dotenv
 import termcolor
 from dateutil import parser
 
 # Configure logging
-logger = logging.getLogger(__name__)
+from lumibot.tools.lumibot_logger import get_logger
+logger = get_logger(__name__)
 
 
 def find_and_load_dotenv(base_dir) -> bool:
@@ -137,18 +137,9 @@ STRATEGY_NAME = os.environ.get("STRATEGY_NAME")
 # Flag to determine if backtest progress should be logged to a file (True/False)
 LOG_BACKTEST_PROGRESS_TO_FILE = os.environ.get("LOG_BACKTEST_PROGRESS_TO_FILE")
 
-# Flag to determine if error logs should be logged to a CSV file (True/False)
-_log_errors_to_csv = os.environ.get("LOG_ERRORS_TO_CSV")
-if _log_errors_to_csv is None:
-    LOG_ERRORS_TO_CSV = False
-elif _log_errors_to_csv.lower() in ("true", "1", "yes", "on"):
-    LOG_ERRORS_TO_CSV = True
-elif _log_errors_to_csv.lower() in ("false", "0", "no", "off"):
-    LOG_ERRORS_TO_CSV = False
-else:
-    LOG_ERRORS_TO_CSV = False
+BACKTESTING_SHOW_PROGRESS_BAR = os.environ.get("BACKTESTING_SHOW_PROGRESS_BAR", "true").lower() == "true"
 
-# Determine if backtesting logs should be quiet via env variable (default True)
+# Determine if backtesting logs should be quiet via env variable (default None means not set)
 _btl = os.environ.get("BACKTESTING_QUIET_LOGS", None)
 if _btl is not None:
     if _btl.lower() == "true":
@@ -156,24 +147,11 @@ if _btl is not None:
     elif _btl.lower() == "false":
         BACKTESTING_QUIET_LOGS = False
     else:
-        colored_message = termcolor.colored(f"BACKTESTING_QUIET_LOGS must be set to 'true' or 'false'. Got '{_btl}'. Defaulting to True.", "yellow")
+        colored_message = termcolor.colored(f"BACKTESTING_QUIET_LOGS must be set to 'true' or 'false'. Got '{_btl}'. Defaulting to None.", "yellow")
         logger.warning(colored_message)
-        BACKTESTING_QUIET_LOGS = True
+        BACKTESTING_QUIET_LOGS = None
 else:
-    BACKTESTING_QUIET_LOGS = None
-
-_btl = os.environ.get("BACKTESTING_SHOW_PROGRESS_BAR", None)
-if _btl is not None:
-    if _btl.lower() == "true":
-        BACKTESTING_SHOW_PROGRESS_BAR = True
-    elif _btl.lower() == "false":
-        BACKTESTING_SHOW_PROGRESS_BAR = False
-    else:
-        colored_message = termcolor.colored(f"BACKTESTING_SHOW_PROGRESS_BAR must be set to 'true' or 'false'. Got '{_btl}'. Defaulting to True.", "yellow")
-        logger.warning(colored_message)
-        BACKTESTING_SHOW_PROGRESS_BAR = True
-else:
-    BACKTESTING_SHOW_PROGRESS_BAR = None
+    BACKTESTING_QUIET_LOGS = True  # Default to quiet logs for better performance
 
 # Set a hard limit on the memory polygon uses
 POLYGON_MAX_MEMORY_BYTES = os.environ.get("POLYGON_MAX_MEMORY_BYTES")
@@ -315,9 +293,55 @@ BITUNIX_CONFIG = {
     "TRADING_MODE": os.environ.get("BITUNIX_TRADING_MODE", "FUTURES"), # Add TRADING_MODE, default to FUTURES
 }
 
+# ProjectX URL mappings - REST API base URLs (v2 gateway URLs preferred)
+PROJECTX_BASE_URLS = {
+    "topstepx": "https://api.topstepx.com/",
+    "topone": "https://api.toponefutures.projectx.com/",  # Top One Futures
+    "tickticktrader": "https://api.tickticktrader.projectx.com/",
+    "alphaticks": "https://api.alphaticks.projectx.com/",
+    "aquafutures": "https://api.aquafutures.projectx.com/",
+    "blueguardianfutures": "https://api.blueguardianfutures.projectx.com/",
+    "blusky": "https://api.blusky.projectx.com/",
+    "bulenox": "https://api.bulenox.projectx.com/",
+    "e8x": "https://api.e8.projectx.com/",  # E8X uses "e8" not "e8x"
+    "fundingfutures": "https://api.fundingfutures.projectx.com/",
+    "thefuturesdesk": "https://api.thefuturesdesk.projectx.com/",
+    "futureselite": "https://api.futureselite.projectx.com/",
+    "fxifyfutures": "https://api.fxifyfutures.projectx.com/",
+    "goatfundedfutures": "https://api.goatfundedfutures.projectx.com/",
+    "holaprime": "https://api.holaprime.projectx.com/",
+    "nexgen": "https://api.nexgen.projectx.com/",
+    "tx3funding": "https://api.tx3funding.projectx.com/",
+    "demo": "https://gateway-api-demo.s2f.projectx.com/",  # Demo still uses old pattern
+    "daytraders": "https://api.daytraders.projectx.com/",
+}
+
+# ProjectX SignalR streaming URL mappings
+PROJECTX_STREAMING_URLS = {
+    "topstepx": "https://gateway-rtc-topstepx.s2f.projectx.com/",
+    "topone": "https://gateway-rtc-demo.s2f.projectx.com/",  # Top One Futures
+    "tickticktrader": "https://gateway-rtc-tickticktrader.s2f.projectx.com/",
+    "alphaticks": "https://gateway-rtc-alphaticks.s2f.projectx.com/",
+    "aquafutures": "https://gateway-rtc-aquafutures.s2f.projectx.com/",
+    "blueguardianfutures": "https://gateway-rtc-blueguardianfutures.s2f.projectx.com/",
+    "blusky": "https://gateway-rtc-blusky.s2f.projectx.com/",
+    "bulenox": "https://gateway-rtc-bulenox.s2f.projectx.com/",
+    "e8x": "https://gateway-rtc-e8x.s2f.projectx.com/",
+    "fundingfutures": "https://gateway-rtc-fundingfutures.s2f.projectx.com/",
+    "thefuturesdesk": "https://gateway-rtc-thefuturesdesk.s2f.projectx.com/",
+    "futureselite": "https://gateway-rtc-futureselite.s2f.projectx.com/",
+    "fxifyfutures": "https://gateway-rtc-fxifyfutures.s2f.projectx.com/",
+    "goatfundedfutures": "https://gateway-rtc-goatfundedfutures.s2f.projectx.com/",
+    "holaprime": "https://gateway-rtc-holaprime.s2f.projectx.com/",
+    "nexgen": "https://gateway-rtc-nexgen.s2f.projectx.com/",
+    "tx3funding": "https://gateway-rtc-tx3funding.s2f.projectx.com/",
+    "demo": "https://gateway-rtc-demo.s2f.projectx.com/",
+    "daytraders": "https://gateway-rtc-daytraders.s2f.projectx.com/",
+}
+
 # ProjectX Configuration - Multi-firm support
 def get_projectx_config(firm: str = None) -> dict:
-    """Get ProjectX configuration for a specific firm"""
+    """Get ProjectX configuration for a specific firm with automatic URL resolution"""
     # If no firm specified, try to get from environment
     if firm is None:
         firm = os.environ.get("PROJECTX_FIRM")
@@ -331,14 +355,23 @@ def get_projectx_config(firm: str = None) -> dict:
     if not firm:
         return {}
     
+    firm_lower = firm.lower()
     firm_upper = firm.upper()
+    
+    # Get URLs: Environment override OR built-in mapping
+    base_url = (os.environ.get(f"PROJECTX_{firm_upper}_BASE_URL") or 
+                PROJECTX_BASE_URLS.get(firm_lower))
+    
+    streaming_url = (os.environ.get(f"PROJECTX_{firm_upper}_STREAMING_BASE_URL") or 
+                     PROJECTX_STREAMING_URLS.get(firm_lower))
+    
     return {
         "firm": firm_upper,
         "api_key": os.environ.get(f"PROJECTX_{firm_upper}_API_KEY"),
         "username": os.environ.get(f"PROJECTX_{firm_upper}_USERNAME"),
-        "base_url": os.environ.get(f"PROJECTX_{firm_upper}_BASE_URL"),
+        "base_url": base_url,
         "preferred_account_name": os.environ.get(f"PROJECTX_{firm_upper}_PREFERRED_ACCOUNT_NAME"),
-        "streaming_base_url": os.environ.get(f"PROJECTX_{firm_upper}_STREAMING_BASE_URL"),
+        "streaming_base_url": streaming_url,
     }
 
 def get_available_projectx_firms() -> list:
@@ -407,6 +440,52 @@ if not is_backtesting or is_backtesting.lower() == "false":
             except Exception as e:
                 colored_message = termcolor.colored(f"Failed to initialize ProjectX broker: {e}", "red")
                 logger.error(colored_message)
+        elif trading_broker_name.lower().startswith("projectx-"):
+            try:
+                # Extract firm name from broker name (e.g., "projectx-topone" -> "topone")
+                firm_suffix = trading_broker_name.lower()[9:]  # Remove "projectx-" prefix
+                
+                # Map broker suffixes to firm names (must match Node.js mapping)
+                suffix_to_firm_mapping = {
+                    'topstepx': 'TOPSTEPX',
+                    'topone': 'TOPONE', 
+                    'tickticktrader': 'TICKTICKTRADER',
+                    'alphaticks': 'ALPHATICKS',
+                    'aquafutures': 'AQUAFUTURES',
+                    'blueguardianfutures': 'BLUEGUARDIANFUTURES',
+                    'blusky': 'BLUSKY',
+                    'bulenox': 'BULENOX',
+                    'e8x': 'E8X',
+                    'fundingfutures': 'FUNDINGFUTURES',
+                    'thefuturesdesk': 'THEFUTURESDESK',
+                    'futureselite': 'FUTURESELITE',
+                    'fxifyfutures': 'FXIFYFUTURES',
+                    'goatfundedfutures': 'GOATFUNDEDFUTURES',
+                    'holaprime': 'HOLAPRIME',
+                    'nexgen': 'NEXGEN',
+                    'tx3funding': 'TX3FUNDING',
+                    'daytraders': 'DAYTRADERS',
+                    'demo': 'DEMO',
+                    # Legacy brokers for backward compatibility
+                    'earn2trade': 'EARN2TRADE',
+                    'uprofit': 'UPROFIT'
+                }
+                
+                if firm_suffix not in suffix_to_firm_mapping:
+                    raise ValueError(f"Unknown ProjectX firm: {firm_suffix}. Supported firms: {list(suffix_to_firm_mapping.keys())}")
+                
+                firm = suffix_to_firm_mapping[firm_suffix]
+                config = get_projectx_config(firm)
+                
+                if not config or not config.get("api_key"):
+                    raise ValueError(f"No valid ProjectX configuration found for firm {firm}. Please set environment variables.")
+                
+                from .data_sources import ProjectXData
+                data_source = ProjectXData(config)
+                broker = ProjectX(config, data_source=data_source)
+            except Exception as e:
+                colored_message = termcolor.colored(f"Failed to initialize ProjectX broker {trading_broker_name}: {e}", "red")
+                logger.error(colored_message)
         else:
             colored_message = termcolor.colored(f"Unknown trading broker name: {trading_broker_name}. Please check your environment variables.", "red")
             logger.error(colored_message)
