@@ -1,49 +1,31 @@
-import logging
-import os
 import datetime as dt
+import os
 from decimal import Decimal
-from typing import Union, Tuple, Optional
-import pytz
-import time
+from typing import Optional, Union
 
 import pandas as pd
-import re
-import requests
-from collections import defaultdict
-from alpaca.data.historical import (
-    CryptoHistoricalDataClient,
-    StockHistoricalDataClient,
-    OptionHistoricalDataClient
-)
+import pytz
+from alpaca.data.enums import Adjustment
+from alpaca.data.historical import CryptoHistoricalDataClient, OptionHistoricalDataClient, StockHistoricalDataClient
 from alpaca.data.requests import (
     CryptoBarsRequest,
-    CryptoLatestQuoteRequest,
-    StockBarsRequest,
-    StockLatestQuoteRequest,
     OptionBarsRequest,
-    OptionLatestTradeRequest,
     OptionChainRequest,
     OptionSnapshotRequest,
+    StockBarsRequest,
 )
 from alpaca.data.timeframe import TimeFrame
-from alpaca.data.enums import Adjustment
 
+from lumibot.constants import LUMIBOT_DEFAULT_QUOTE_ASSET_SYMBOL, LUMIBOT_DEFAULT_QUOTE_ASSET_TYPE
 from lumibot.entities import Asset, Bars, Quote
-from lumibot import (
-    LUMIBOT_DEFAULT_TIMEZONE,
-    LUMIBOT_DEFAULT_QUOTE_ASSET_SYMBOL,
-    LUMIBOT_DEFAULT_QUOTE_ASSET_TYPE
-)
-from lumibot.tools.helpers import (
-    get_decimals,
-    quantize_to_num_decimals,
-    get_trading_days,
-    date_n_trading_days_from_date
-)
 from lumibot.tools.alpaca_helpers import sanitize_base_and_quote_asset
+from lumibot.tools.helpers import date_n_trading_days_from_date
+from lumibot.tools.lumibot_logger import get_logger
 
 from .data_source import DataSource
 
+logger = get_logger(__name__)
+
 
 class AlpacaData(DataSource):
     SOURCE = "ALPACA"
@@ -120,7 +102,18 @@ class AlpacaData(DataSource):
         This will mark the strategy as failed to stop execution.
         """
         error_message = str(e).lower()
-        if "unauthorized" in error_message or "401" in error_message:
+        # Only treat specific authentication-related errors as auth failures
+        is_auth_error = (
+            "unauthorized" in error_message or 
+            "401" in error_message or
+            "403" in error_message or
+            "invalid credentials" in error_message or
+            "authentication failed" in error_message or
+            "invalid api key" in error_message or
+            "invalid token" in error_message
+        )
+        
+        if is_auth_error:
             auth_method = "OAuth token" if self.oauth_token else "API key/secret"
             error_msg = (
                 f"❌ ALPACA AUTHENTICATION ERROR: Your {auth_method} appears to be invalid or expired during {operation}.\n\n"
@@ -141,12 +134,12 @@ class AlpacaData(DataSource):
                 )
             else:
                 error_msg += (
-                    f"1. Check that your ALPACA_API_KEY and ALPACA_API_SECRET environment variables are set correctly\n"
-                    f"2. Verify your API credentials are valid\n"
-                    f"3. Check that your account has proper data permissions\n\n"
+                    "1. Check that your ALPACA_API_KEY and ALPACA_API_SECRET environment variables are set correctly\n"
+                    "2. Verify your API credentials are valid\n"
+                    "3. Check that your account has proper data permissions\n\n"
                 )
             error_msg += f"💀 STOPPING STRATEGY EXECUTION\n\nOriginal error: {e}"
-            logging.error(error_msg)
+            logger.error(error_msg)
 
             # Mark the data source as failed to stop further requests
             self._auth_failed = True
@@ -154,6 +147,8 @@ class AlpacaData(DataSource):
             # Raise a regular exception that will be caught by the strategy
             raise ValueError(f"Authentication failed: {auth_method} is invalid or expired. {error_msg}")
         else:
+            # For non-auth errors, log the error but don't mark as auth failed
+            logger.warning(f"Non-authentication error during {operation}: {e}")
             # Re-raise the original exception for other errors
             raise e
 
@@ -166,7 +161,17 @@ class AlpacaData(DataSource):
                 else:
                     self._stock_client = StockHistoricalDataClient(self.api_key, self.api_secret)
             except Exception as e:
-                self._handle_auth_error(e, "stock client initialization")
+                # Check if this is specifically an authentication error
+                error_message = str(e).lower()
+                if any(auth_keyword in error_message for auth_keyword in [
+                    "unauthorized", "401", "403", "invalid credentials", 
+                    "authentication failed", "invalid api key", "invalid token"
+                ]):
+                    self._handle_auth_error(e, "stock client initialization")
+                else:
+                    # For other errors, log and re-raise without marking as auth failed
+                    logger.warning(f"Error initializing stock client (will retry): {e}")
+                    raise e
         return self._stock_client
 
     def _get_crypto_client(self):
@@ -178,7 +183,17 @@ class AlpacaData(DataSource):
                 else:
                     self._crypto_client = CryptoHistoricalDataClient(self.api_key, self.api_secret)
             except Exception as e:
-                self._handle_auth_error(e, "crypto client initialization")
+                # Check if this is specifically an authentication error
+                error_message = str(e).lower()
+                if any(auth_keyword in error_message for auth_keyword in [
+                    "unauthorized", "401", "403", "invalid credentials", 
+                    "authentication failed", "invalid api key", "invalid token"
+                ]):
+                    self._handle_auth_error(e, "crypto client initialization")
+                else:
+                    # For other errors, log and re-raise without marking as auth failed
+                    logger.warning(f"Error initializing crypto client (will retry): {e}")
+                    raise e
         return self._crypto_client
 
     def _get_option_client(self):
@@ -190,7 +205,18 @@ class AlpacaData(DataSource):
                 else:
                     self._option_client = OptionHistoricalDataClient(self.api_key, self.api_secret)
             except Exception as e:
-                self._handle_auth_error(e, "option client initialization")
+                # Log the actual error without going through auth error handler immediately
+                logger.error(f"Error initializing option client: {e}")
+                # Only call auth error handler for actual auth errors
+                error_message = str(e).lower()
+                if any(auth_keyword in error_message for auth_keyword in [
+                    "unauthorized", "401", "403", "invalid credentials", 
+                    "authentication failed", "invalid api key", "invalid token"
+                ]):
+                    self._handle_auth_error(e, "option client initialization")
+                else:
+                    # For other errors, just re-raise so the actual error is visible
+                    raise e
         return self._option_client
 
     def __init__(
@@ -257,13 +283,8 @@ class AlpacaData(DataSource):
         self.oauth_token = None
         self._auth_failed = False  # Flag to track authentication failures
 
-        # Check for OAuth token first
-        if isinstance(config, dict) and "OAUTH_TOKEN" in config and config["OAUTH_TOKEN"]:
-            self.oauth_token = config["OAUTH_TOKEN"]
-        elif hasattr(config, "OAUTH_TOKEN") and config.OAUTH_TOKEN:
-            self.oauth_token = config.OAUTH_TOKEN
-        # If no OAuth token, check for API key/secret
-        elif isinstance(config, dict) and "API_KEY" in config and config["API_KEY"]:
+        # Check for API key/secret first (prefer API keys over OAuth tokens)
+        if isinstance(config, dict) and "API_KEY" in config and config["API_KEY"]:
             self.api_key = config["API_KEY"]
             if "API_SECRET" in config and config["API_SECRET"]:
                 self.api_secret = config["API_SECRET"]
@@ -275,13 +296,18 @@ class AlpacaData(DataSource):
                 self.api_secret = config.API_SECRET
             else:
                 raise ValueError("API_SECRET not found in config when API_KEY is provided")
+        # If no API key/secret, check for OAuth token
+        elif isinstance(config, dict) and "OAUTH_TOKEN" in config and config["OAUTH_TOKEN"]:
+            self.oauth_token = config["OAUTH_TOKEN"]
+        elif hasattr(config, "OAUTH_TOKEN") and config.OAUTH_TOKEN:
+            self.oauth_token = config.OAUTH_TOKEN
         else:
             raise ValueError("Either OAuth token or API key/secret must be provided for Alpaca authentication")
 
         # If an ENDPOINT is provided, warn the user that it is not used anymore
         # Instead they should use the "PAPER" parameter, which is boolean
         if isinstance(config, dict) and "ENDPOINT" in config:
-            logging.warning(
+            logger.warning(
                 """The ENDPOINT parameter is not used anymore for AlpacaData, please use the PAPER parameter instead.
                 The 'PAPER' parameter is boolean, and defaults to True.
                 The ENDPOINT parameter will be removed in a future version of lumibot."""
@@ -302,6 +328,17 @@ class AlpacaData(DataSource):
         else:
             self.version = "v2"
 
+    def reset_auth_failure(self):
+        """
+        Reset the authentication failure state and clear cached clients.
+        This allows the data source to retry authentication after a failure.
+        """
+        self._auth_failed = False
+        self._stock_client = None
+        self._crypto_client = None
+        self._option_client = None
+        logger.info("Authentication failure state has been reset - will retry API calls")
+
     def _sanitize_base_and_quote_asset(self, base_asset, quote_asset) -> tuple[Asset, Asset]:
         asset, quote = sanitize_base_and_quote_asset(base_asset, quote_asset)
         return asset, quote
@@ -334,7 +371,13 @@ class AlpacaData(DataSource):
         """
         # Check if authentication has previously failed
         if getattr(self, '_auth_failed', False):
-            raise ValueError("Authentication previously failed - cannot make further API requests")
+            logger.warning("Authentication failure flag is set - attempting to clear and retry")
+            # Instead of immediately failing, reset the flag and let the actual error show
+            self._auth_failed = False
+            # Clear the cached clients so they get recreated
+            self._stock_client = None
+            self._crypto_client = None
+            self._option_client = None
 
         try:
             # Use the existing option client getter which has proper error handling
@@ -374,7 +417,7 @@ class AlpacaData(DataSource):
                     option_symbols = [key for key in raw_chain_data.keys() if key.startswith(asset.symbol) and len(key) > len(asset.symbol)]
 
             if not option_symbols:
-                logging.warning(f"No option symbols found for {asset.symbol}")
+                logger.warning(f"No option symbols found for {asset.symbol}")
                 return chains_data
 
             # Parse each option symbol
@@ -382,7 +425,7 @@ class AlpacaData(DataSource):
             for symbol in option_symbols:
                 # Parse option symbol to extract details
                 # Alpaca option symbols format: SPYYYMMDDCPPPPPPPPP
-                # Where: SPY = underlying, YY = year, MM = month, DD = day, 
+                # Where: SPY = underlying, YY = year, MM = month, DD = day,
                 #        C/P = call/put, PPPPPPPPP = strike price (padded)
 
                 if len(symbol) < 15:  # Skip invalid symbols
@@ -436,13 +479,44 @@ class AlpacaData(DataSource):
                 for expiration_date in chains_data["Chains"][option_type]:
                     chains_data["Chains"][option_type][expiration_date].sort()
 
-            logging.debug(f"Successfully retrieved option chains for {asset.symbol}: {len(chains_data['Chains']['PUT'])} PUT expirations, {len(chains_data['Chains']['CALL'])} CALL expirations")
+            logger.debug(f"Successfully retrieved option chains for {asset.symbol}: {len(chains_data['Chains']['PUT'])} PUT expirations, {len(chains_data['Chains']['CALL'])} CALL expirations")
 
             return chains_data
 
         except Exception as e:
-            # Handle any additional errors not caught by client initialization
-            self._handle_auth_error(e, "option chain retrieval")
+            # Log the actual error first so we can see what's really happening
+            logger.error(f"Error retrieving option chains for {asset.symbol}: {e}")
+            logger.error(f"Error type: {type(e).__name__}")
+            logger.error(f"Full error details: {str(e)}")
+            
+            # Check if this is specifically an authentication error
+            error_message = str(e).lower()
+            
+            # Be more specific about what constitutes an auth error
+            # Don't treat every 401 as an auth error - could be data permissions, rate limits, etc.
+            is_likely_auth_error = (
+                ("unauthorized" in error_message and (
+                    "invalid" in error_message or 
+                    "expired" in error_message or 
+                    "credentials" in error_message or
+                    "api key" in error_message or
+                    "token" in error_message
+                )) or
+                "authentication failed" in error_message or
+                "invalid api key" in error_message or
+                "invalid token" in error_message or
+                "invalid credentials" in error_message
+            )
+            
+            if is_likely_auth_error:
+                logger.error("This appears to be an authentication error - handling as auth failure")
+                # Handle authentication errors which will set _auth_failed flag
+                self._handle_auth_error(e, "option chain retrieval")
+            else:
+                # For other errors (network, rate limits, data permissions, etc.), just re-raise the original error
+                # This ensures the user sees the actual error, not a generic auth message
+                logger.error("This does not appear to be an authentication error - re-raising original error")
+                raise e
 
     def get_last_price(self, asset, quote=None, exchange=None, **kwargs) -> Union[float, Decimal, None]:
         """
@@ -465,13 +539,14 @@ class AlpacaData(DataSource):
             timeshift: Optional[dt.timedelta] = None,
             quote: Optional[Asset] = None,
             exchange: Optional[str] = None,
-            include_after_hours: bool = True
+            include_after_hours: bool = True,
+            return_polars: bool = False,
     ) -> Optional[Bars]:
 
         """Get bars for a given asset"""
 
         if exchange is not None:
-            logging.warning(
+            logger.warning(
                 f"the exchange parameter is not implemented for AlpacaData, but {exchange} was passed as the exchange"
             )
 
@@ -507,7 +582,7 @@ class AlpacaData(DataSource):
 
         timeframe = self._parse_source_timestep(timestep, reverse=True)
 
-        now = dt.datetime.now(self._tzinfo)
+        now = dt.datetime.now(self.tzinfo)
 
         # Create end time
         if asset.asset_type != Asset.AssetType.CRYPTO and isinstance(self._delay, dt.timedelta):
@@ -536,7 +611,7 @@ class AlpacaData(DataSource):
             # This works for now. Crypto gets more bars but throws them out.
             market='NYSE'
         )
-        start_dt = self._tzinfo.localize(dt.datetime.combine(start_date, dt.datetime.min.time()))
+        start_dt = self.tzinfo.localize(dt.datetime.combine(start_date, dt.datetime.min.time()))
 
         # Make API request based on asset type
         try:
@@ -585,12 +660,12 @@ class AlpacaData(DataSource):
             df = barset.df
 
         except Exception as e:
-            logging.error(f"Could not get pricing data from Alpaca for {symbol} with error: {e}")
+            logger.error(f"Could not get pricing data from Alpaca for {symbol} with error: {e}")
             return None
 
         # Handle case where no data was received
         if df.empty:
-            logging.warning(f"No pricing data available from Alpaca for {symbol}")
+            logger.warning(f"No pricing data available from Alpaca for {symbol}")
             return None
 
         # Remove MultiIndex
@@ -599,16 +674,16 @@ class AlpacaData(DataSource):
         # Timezone conversion
         if hasattr(df.index, 'tz'):
             if df.index.tz is not None:
-                df.index = df.index.tz_convert(self._tzinfo)
+                df.index = df.index.tz_convert(self.tzinfo)
             else:
-                df.index = self._tzinfo.localize(df.index)
+                df.index = self.tzinfo.localize(df.index)
 
         # Clean up the dataframe
         df = df[~df.index.duplicated(keep="first")]
         df = df.sort_index()
         df = df[df.close > 0]
 
-        if not include_after_hours and timestep == 'minute' and self._tzinfo == pytz.timezone("America/New_York"):
+        if not include_after_hours and timestep == 'minute' and self.tzinfo == pytz.timezone("America/New_York"):
             # Filter data to include only regular market hours
             df = df[(df.index.hour >= 9) & (df.index.minute >= 30) & (df.index.hour < 16)]
 
@@ -641,7 +716,13 @@ class AlpacaData(DataSource):
 
         # Check if authentication has previously failed
         if getattr(self, '_auth_failed', False):
-            raise ValueError("Authentication previously failed - cannot make further API requests")
+            logger.warning("Authentication failure flag is set - attempting to clear and retry")
+            # Instead of immediately failing, reset the flag and let the actual error show
+            self._auth_failed = False
+            # Clear the cached clients so they get recreated
+            self._stock_client = None
+            self._crypto_client = None
+            self._option_client = None
 
         asset, quote = self._sanitize_base_and_quote_asset(asset, quote)
         if asset.asset_type == Asset.AssetType.CRYPTO:
@@ -757,5 +838,5 @@ class AlpacaData(DataSource):
                 'rho': greeks_obj.rho,
             }
         except Exception as e:
-            logging.error(f"Error fetching greeks from Alpaca Data API: {e}")
+            logger.error(f"Error fetching greeks from Alpaca Data API: {e}")
             return {}
