@@ -1,22 +1,24 @@
-import logging
 import os
+import time
+import traceback
 from abc import ABC, abstractmethod
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from datetime import datetime, timedelta
-import traceback
-import time
 from decimal import Decimal
 from typing import Union
-import pytz
 
 import pandas as pd
+import pytz
 
-from lumibot import LUMIBOT_DEFAULT_PYTZ, LUMIBOT_DEFAULT_TIMEZONE
+from lumibot.constants import LUMIBOT_DEFAULT_PYTZ, LUMIBOT_DEFAULT_TIMEZONE
 from lumibot.entities import Asset, AssetsMapping, Bars, Quote
 from lumibot.tools import black_scholes, create_options_symbol
+from lumibot.tools.lumibot_logger import get_logger
 
 from .exceptions import UnavailabeTimestep
 
+logger = get_logger(__name__)
+
 
 class DataSource(ABC):
     SOURCE = ""
@@ -64,7 +66,10 @@ class DataSource(ABC):
 
         if tzinfo is None:
             tzinfo = pytz.timezone(self.DEFAULT_TIMEZONE)
-        self._tzinfo = tzinfo
+        self.tzinfo = tzinfo
+
+        # Initialize caches centrally (avoid ad-hoc hasattr checks in methods)
+        self._greeks_cache = {}
 
     # ========Required Implementations ======================
     @abstractmethod
@@ -96,7 +101,7 @@ class DataSource(ABC):
 
     @abstractmethod
     def get_historical_prices(
-        self, asset, length, timestep="", timeshift=None, quote=None, exchange=None, include_after_hours=True
+        self, asset, length, timestep="", timeshift=None, quote=None, exchange=None, include_after_hours=True, return_polars=False
     ) -> Bars:
         """
         Get bars for a given asset, going back in time from now, getting length number of bars by timestep.
@@ -126,6 +131,8 @@ class DataSource(ABC):
             The exchange to get the bars for.
         include_after_hours : bool
             Whether to include after hours data.
+        return_polars : bool
+            If True, return Bars with Polars DataFrame for better performance. Default is False (returns pandas).
 
         Returns
         -------
@@ -241,10 +248,10 @@ class DataSource(ABC):
         if dt.tzinfo is not None and dt.tzinfo.utcoffset(dt) is not None:
             return self.to_default_timezone(dt)
         else:
-            return self._tzinfo.localize(dt, is_dst=None)
+            return self.tzinfo.localize(dt, is_dst=None)
 
     def to_default_timezone(self, dt):
-        return dt.astimezone(self._tzinfo)
+        return dt.astimezone(self.tzinfo)
 
     def get_timestep(self):
         return self._timestep if self._timestep else self.MIN_TIMESTEP
@@ -374,9 +381,9 @@ class DataSource(ABC):
                     time.sleep(0.1)
                 except Exception as e:
                     # Log once per asset to avoid spamming with a huge traceback
-                    logging.warning(f"Error retrieving data for {base_asset.symbol}: {e}")
+                    logger.warning(f"Error retrieving data for {base_asset.symbol}: {e}")
                     tb = traceback.format_exc()
-                    logging.warning(tb)  # This prints the traceback
+                    logger.warning(tb)  # This prints the traceback
                     chunk_result[asset] = None
             return chunk_result
 
@@ -518,7 +525,7 @@ class DataSource(ABC):
                 row.update({f"greeks.{col}": val for col, val in greeks.items()})
                 rows.append(row)
 
-        logging.info(f"Chain Full Info Query Total: {query_total:.2f}s. "
+        logger.info(f"Chain Full Info Query Total: {query_total:.2f}s. "
                      f"Total Time: {time.perf_counter() - start_t:.2f}s, "
                      f"Rows: {len(rows)}")
         return pd.DataFrame(rows).sort_values("strike") if rows else pd.DataFrame()
@@ -532,10 +539,38 @@ class DataSource(ABC):
         risk_free_rate: float,
     ):
         """Returns Greeks in backtesting."""
+        # Handle None values - don't cache or calculate if inputs are invalid
+        if asset_price is None or underlying_price is None or risk_free_rate is None:
+            return None
+
+        # Optimization: Cache Greeks calculations based on key parameters
+        # Round prices to 2 decimal places for cache key to handle minor price fluctuations
+        current_date = self.get_datetime()
+        cache_key = (
+            asset.symbol,
+            asset.strike,
+            asset.right,
+            asset.expiration,
+            round(asset_price, 2),
+            round(underlying_price, 2),
+            round(risk_free_rate, 4),
+            current_date.date() if hasattr(current_date, 'date') else current_date  # Cache per day to handle time decay
+        )
+
+        # Check cache
+        if cache_key in self._greeks_cache:
+            return self._greeks_cache[cache_key]
+
+        # Keep cache size limited to prevent memory issues
+        if len(self._greeks_cache) > 10000:
+            # Clear oldest half of cache
+            keys_to_remove = list(self._greeks_cache.keys())[:5000]
+            for key in keys_to_remove:
+                del self._greeks_cache[key]
+
         opt_price = asset_price
         und_price = underlying_price
         interest = risk_free_rate * 100
-        current_date = self.get_datetime()
 
         # If asset expiration is a datetime object, convert it to date
         expiration = asset.expiration
@@ -544,8 +579,8 @@ class DataSource(ABC):
 
         # Convert the expiration to be a datetime with 4pm New York time
         expiration = datetime.combine(expiration, datetime.min.time())
-        expiration = self._tzinfo.localize(expiration)
-        expiration = expiration.astimezone(self._tzinfo)
+        expiration = self.tzinfo.localize(expiration)
+        expiration = expiration.astimezone(self.tzinfo)
         expiration = expiration.replace(hour=16, minute=0, second=0, microsecond=0)
 
         # Calculate the days to expiration, but allow for fractional days
@@ -582,11 +617,14 @@ class DataSource(ABC):
             underlying_price=und_price,
         )
 
+        # Cache the result
+        self._greeks_cache[cache_key] = greeks
+
         return greeks
 
     def query_greeks(self, asset):
         """Query for the Greeks as it can be more accurate than calculating locally."""
-        logging.info(f"Querying Options Greeks for {asset.symbol} is not supported for this "
+        logger.info(f"Querying Options Greeks for {asset.symbol} is not supported for this "
                      f"data source {self.__class__}.")
         return {}
 
