@@ -1,8 +1,11 @@
-import logging
 import re
 import time
 from threading import currentThread
 
+from lumibot.tools.lumibot_logger import get_logger
+
+logger = get_logger(__name__)
+
 default_time_sleep = time.sleep
 warned_against_calling_time_sleep = False
 
@@ -18,7 +21,7 @@ def warning_time_sleep(sleeptime):
             warned_against_calling_time_sleep = True
             # TODO: Look into this warning being handled more gracefully. Right now it
             # is being called too often
-            # logging.critical(
+            # logger.critical(
             #     "Time.sleep has been called within thread %s."
             #     "time.sleep should be used with caution within lumibot, "
             #     "especially in backtesting mode." % thread_name
