@@ -4,6 +4,7 @@ from datetime import datetime, timedelta, timezone
 import pandas as pd
 
 from lumibot.tools import databento_helper
+from lumibot.tools.databento_helper import DataBentoClient
 from lumibot.entities import Asset
 
 
@@ -29,14 +30,16 @@ class TestDataBentoHelper(unittest.TestCase):
         """Test futures symbol formatting"""
         # Test continuous futures (CONT_FUTURE) - should resolve to specific contract
         continuous_asset = Asset(symbol="ES", asset_type=Asset.AssetType.CONT_FUTURE)
-        result = databento_helper._format_futures_symbol_for_databento(continuous_asset)
-        # For July 2025, should resolve to September contract (ESU5 in DataBento format)
-        self.assertIn("ESU5", result)
         
-        # Test MES continuous futures
+        # Test with specific reference date (January 1, 2025) - should resolve to March contract
+        reference_date = datetime(2025, 1, 1)
+        result = databento_helper._format_futures_symbol_for_databento(continuous_asset, reference_date)
+        self.assertIn("ESH25", result)
+        
+        # Test MES continuous futures with same reference date
         mes_continuous = Asset(symbol="MES", asset_type=Asset.AssetType.CONT_FUTURE)
-        result = databento_helper._format_futures_symbol_for_databento(mes_continuous)
-        self.assertIn("MESU5", result)
+        result = databento_helper._format_futures_symbol_for_databento(mes_continuous, reference_date)
+        self.assertIn("MESH25", result)
         
         # Test regular future (no expiration) - should return raw symbol
         regular_future = Asset(symbol="ES", asset_type="future")
@@ -215,7 +218,7 @@ class TestDataBentoHelper(unittest.TestCase):
             "minute"
         )
         
-        expected_name = "ES_20250315_minute_20250101_20250131.feather"
+        expected_name = "ES_20250315_minute_20250101_20250131.parquet"
         self.assertEqual(filename.name, expected_name)
 
     @patch('lumibot.tools.databento_helper.DATABENTO_AVAILABLE', True)
@@ -413,7 +416,7 @@ class TestDataBentoHelper(unittest.TestCase):
                 )
                 
                 # Should be a valid filename
-                self.assertTrue(filename.name.endswith('.feather'))
+                self.assertTrue(filename.name.endswith('.parquet'))
                 self.assertNotIn('/', filename.name)  # No path separators
                 self.assertNotIn('\\', filename.name)  # No path separators
                 
@@ -449,21 +452,16 @@ class TestDataBentoHelper(unittest.TestCase):
         # Extra columns should be preserved
         self.assertIn('extra_column', result.columns)
 
-    def test_asset_class_method_delegation(self):
-        """Test that DataBento helper properly delegates to Asset class methods"""
-        # Test that continuous futures use Asset class resolution
+    def test_continuous_futures_resolution(self):
+        """Test that DataBento helper properly resolves continuous futures"""
+        # Test that continuous futures use internal resolution logic
         continuous_asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
         
-        # Mock the Asset class method to ensure it's being called
-        with patch.object(continuous_asset, 'resolve_continuous_futures_contract', 
-                         return_value='MESU25') as mock_resolve:
-            
-            result = databento_helper._format_futures_symbol_for_databento(continuous_asset)
-            
-            # Should have called the Asset class method
-            mock_resolve.assert_called_once()
-            # Result should be processed through DataBento symbol alternatives (MESU25 -> MESU5)
-            self.assertEqual(result, 'MESU5')
+        result = databento_helper._format_futures_symbol_for_databento(continuous_asset)
+        
+        # Should return a resolved contract with month code and year
+        self.assertIn("MES", result)
+        self.assertTrue(len(result) > 3)  # Should have month code and year
         
         # Test that specific futures don't use continuous resolution
         specific_asset = Asset("MES", asset_type="future", expiration=datetime(2025, 9, 15).date())
@@ -619,8 +617,8 @@ class TestDataBentoHelper(unittest.TestCase):
 
     @patch('lumibot.tools.databento_helper.DATABENTO_AVAILABLE', True)
     @patch('lumibot.tools.databento_helper.DataBentoClient')
-    def test_get_price_data_resolves_once_no_retry(self, mock_client_class):
-        """Test that get_price_data_from_databento resolves continuous futures once, no retry"""
+    def test_get_price_data_continuous_futures_resolution(self, mock_client_class):
+        """Test that get_price_data_from_databento resolves continuous futures using internal logic"""
         
         # Mock the DataBento client
         mock_client = MagicMock()
@@ -632,28 +630,22 @@ class TestDataBentoHelper(unittest.TestCase):
         # Create continuous futures asset
         es_cont = Asset(symbol="ES", asset_type="cont_future")
         
-        # Mock the asset's resolve method to return a specific contract
-        with patch.object(es_cont, 'resolve_continuous_futures_contract', return_value='ESH25'):
-            
-            # Call the function
-            result = databento_helper.get_price_data_from_databento(
-                api_key="test_api_key",
-                asset=es_cont,
-                start=datetime(2025, 1, 1),
-                end=datetime(2025, 1, 2),
-                timestep="minute"
-            )
-            
-            # Verify that resolve_continuous_futures_contract was called exactly once
-            es_cont.resolve_continuous_futures_contract.assert_called_once()
-            
-            # Verify that DataBento client get_historical_data was called exactly once
-            # (no retry logic should mean only one call)
-            self.assertEqual(mock_client.get_historical_data.call_count, 1)
-            
-            # Verify the specific call was made with the resolved symbol
-            call_args = mock_client.get_historical_data.call_args
-            self.assertIn('ESH5', call_args[1]['symbols'])  # symbols parameter should contain ESH5
+        # Call the function
+        result = databento_helper.get_price_data_from_databento(
+            api_key="test_api_key",
+            asset=es_cont,
+            start=datetime(2025, 1, 1),
+            end=datetime(2025, 1, 2),
+            timestep="minute"
+        )
+        
+        # Verify that DataBento client get_historical_data was called
+        self.assertGreater(mock_client.get_historical_data.call_count, 0)
+        
+        # Verify the specific call was made with the resolved symbol
+        call_args = mock_client.get_historical_data.call_args
+        # Should contain the resolved contract (ESH5 for January 2025 using DataBento short year format)
+        self.assertIn('ESH5', str(call_args))
 
     @patch('lumibot.tools.databento_helper.DATABENTO_AVAILABLE', True)
     @patch('lumibot.tools.databento_helper.Historical')
@@ -709,3 +701,313 @@ class TestDataBentoHelper(unittest.TestCase):
         # Verify that resolve_continuous_futures_contract is used instead
         self.assertIn('resolve_continuous_futures_contract', source,
                       "DataBento helper should use resolve_continuous_futures_contract method")
+
+    def test_market_calendar_spanning_sessions(self):
+        """Test market calendar logic for sessions spanning multiple days."""
+        from unittest.mock import MagicMock
+        from lumibot.brokers.broker import Broker
+        
+        # Create a mock broker
+        broker = MagicMock(spec=Broker)
+        broker.market = "CME_Equity"
+        
+        # Mock the market_hours method to simulate CME futures schedule
+        def mock_market_hours(close=True, next=False):
+            if next:  # Friday's session
+                if close:
+                    return datetime(2025, 1, 10, 23, 0, tzinfo=timezone.utc)  # 6pm ET Fri
+                else:
+                    return datetime(2025, 1, 9, 23, 0, tzinfo=timezone.utc)   # 6pm ET Thu
+            else:  # Thursday's session  
+                if close:
+                    return datetime(2025, 1, 9, 23, 0, tzinfo=timezone.utc)   # 6pm ET Thu
+                else:
+                    return datetime(2025, 1, 8, 23, 0, tzinfo=timezone.utc)   # 6pm ET Wed
+        
+        def mock_utc_to_local(utc_time):
+            # Convert UTC to ET (UTC-5) and return as naive datetime
+            from dateutil import tz
+            return utc_time.replace(tzinfo=timezone.utc).astimezone(tz=tz.tzlocal()).replace(tzinfo=None)
+        
+        broker.market_hours.side_effect = mock_market_hours
+        broker.utc_to_local.side_effect = mock_utc_to_local
+        
+        # Test Thursday 7pm ET (should be open - Friday's session)
+        current_time = datetime(2025, 1, 9, 19, 0, 0)  # 7pm ET Thursday
+        
+        # Simulate the logic from is_market_open
+        result_today = False
+        result_tomorrow = False
+        
+        # Check today's session (Thursday)
+        try:
+            open_time_today = mock_utc_to_local(mock_market_hours(close=False, next=False))
+            close_time_today = mock_utc_to_local(mock_market_hours(close=True, next=False))
+            
+            if (current_time >= open_time_today) and (close_time_today >= current_time):
+                result_today = True
+        except:
+            result_today = False
+        
+        # Check tomorrow's session (Friday)
+        try:
+            open_time_tomorrow = mock_utc_to_local(mock_market_hours(close=False, next=True))
+            close_time_tomorrow = mock_utc_to_local(mock_market_hours(close=True, next=True))
+            
+            if (current_time >= open_time_tomorrow) and (close_time_tomorrow >= current_time):
+                result_tomorrow = True
+        except:
+            result_tomorrow = False
+        
+        is_open = result_today or result_tomorrow
+        
+        # 7pm Thursday should be market open (Friday's session started at 6pm Thursday)
+        self.assertTrue(is_open, "CME futures should be open at 7pm Thursday ET (Friday's session)")
+
+
+class TestDataBentoAuthenticationRetry(unittest.TestCase):
+    """Test cases for DataBento authentication retry logic"""
+
+    def setUp(self):
+        """Set up test fixtures"""
+        self.api_key = "test_api_key"
+        self.client = DataBentoClient(self.api_key)
+
+    @patch('lumibot.tools.databento_helper.Historical')
+    def test_authentication_retry_on_401_error(self, mock_historical):
+        """Test that authentication errors trigger client recreation and retry"""
+        
+        # First call fails with 401, second succeeds
+        mock_historical.return_value.timeseries.get_range.side_effect = [
+            Exception("401 auth_authentication_failed\nAuthentication failed."),
+            Mock(to_df=Mock(return_value=pd.DataFrame({'test': [1, 2, 3]})))
+        ]
+        
+        with patch('lumibot.tools.databento_helper.Historical', mock_historical):
+            result = self.client.get_historical_data(
+                dataset="GLBX.MDP3",
+                symbols="MESU5",
+                schema="ohlcv-1m", 
+                start="2025-01-01",
+                end="2025-01-02"
+            )
+            
+            # Should succeed on retry
+            self.assertIsInstance(result, pd.DataFrame)
+            self.assertEqual(len(result), 3)
+            
+            # Should have called Historical constructor twice (original + retry)
+            self.assertEqual(mock_historical.call_count, 2)
+            
+            # Should have called get_range twice (first fails, second succeeds)
+            self.assertEqual(mock_historical.return_value.timeseries.get_range.call_count, 2)
+
+    @patch('lumibot.tools.databento_helper.Historical')
+    def test_authentication_retry_exhausts_after_max_retries(self, mock_historical):
+        """Test that authentication retries are exhausted after max attempts"""
+        
+        # All calls fail with 401
+        mock_historical.return_value.timeseries.get_range.side_effect = Exception(
+            "401 auth_authentication_failed\nAuthentication failed."
+        )
+        
+        with patch('lumibot.tools.databento_helper.Historical', mock_historical):
+            with self.assertRaises(Exception) as context:
+                self.client.get_historical_data(
+                    dataset="GLBX.MDP3",
+                    symbols="MESU5", 
+                    schema="ohlcv-1m",
+                    start="2025-01-01",
+                    end="2025-01-02"
+                )
+            
+            # Should contain authentication error
+            self.assertIn("auth_authentication_failed", str(context.exception))
+            
+            # Should have tried max_retries times (default 3) + initial creation = 4
+            self.assertEqual(mock_historical.call_count, 4)
+
+    @patch('lumibot.tools.databento_helper.Historical')
+    def test_non_auth_errors_not_retried(self, mock_historical):
+        """Test that non-authentication errors are not retried"""
+        
+        # Non-auth error should not trigger retry
+        mock_historical.return_value.timeseries.get_range.side_effect = Exception("Invalid symbol")
+        
+        with self.assertRaises(Exception) as context:
+            self.client.get_historical_data(
+                dataset="GLBX.MDP3",
+                symbols="INVALID",
+                schema="ohlcv-1m",
+                start="2025-01-01", 
+                end="2025-01-02"
+            )
+        
+        # Should contain original error
+        self.assertIn("Invalid symbol", str(context.exception))
+        
+        # Should only have tried once (no retry for non-auth errors)
+        self.assertEqual(mock_historical.call_count, 1)
+
+    @patch('lumibot.tools.databento_helper.Historical')
+    def test_403_forbidden_triggers_retry(self, mock_historical):
+        """Test that 403 Forbidden errors also trigger authentication retry"""
+        
+        # 403 error should also trigger retry
+        mock_historical.return_value.timeseries.get_range.side_effect = [
+            Exception("403 Forbidden"),
+            Mock(to_df=Mock(return_value=pd.DataFrame({'data': [1]})))
+        ]
+        
+        with patch('lumibot.tools.databento_helper.Historical', mock_historical):
+            result = self.client.get_historical_data(
+                dataset="GLBX.MDP3",
+                symbols="MESU5",
+                schema="ohlcv-1m",
+                start="2025-01-01",
+                end="2025-01-02"
+            )
+            
+            # Should succeed on retry
+            self.assertIsInstance(result, pd.DataFrame)
+            
+            # Should have recreated client
+            self.assertEqual(mock_historical.call_count, 2)
+
+    @patch('lumibot.tools.databento_helper.Historical')
+    def test_client_recreation_on_auth_failure(self, mock_historical):
+        """Test that client is properly recreated on authentication failure"""
+        
+        # Create client and access it to initialize
+        original_client = self.client.client
+        self.assertIsNotNone(original_client)
+        
+        # Simulate auth failure
+        mock_historical.return_value.timeseries.get_range.side_effect = Exception("401 auth_authentication_failed")
+        
+        with patch('lumibot.tools.databento_helper.Historical', mock_historical):
+            try:
+                self.client.get_historical_data(
+                    dataset="GLBX.MDP3",
+                    symbols="MESU5", 
+                    schema="ohlcv-1m",
+                    start="2025-01-01",
+                    end="2025-01-02"
+                )
+            except Exception:
+                pass  # Expected to fail after retries
+            
+            # Client should have been recreated during retry attempts  
+            self.assertEqual(mock_historical.call_count, 4)  # initial + max_retries = 4
+
+    @patch('lumibot.tools.databento_helper.Historical')
+    def test_successful_retry_after_token_refresh(self, mock_historical):
+        """Test successful operation after token refresh simulation"""
+        
+        # Simulate token expiry then refresh success
+        responses = [
+            Exception("401 auth_authentication_failed"),  # First call fails
+            Exception("401 auth_authentication_failed"),  # Second call fails  
+            Mock(to_df=Mock(return_value=pd.DataFrame({'success': [1, 2]})))  # Third succeeds
+        ]
+        mock_historical.return_value.timeseries.get_range.side_effect = responses
+        
+        with patch('lumibot.tools.databento_helper.Historical', mock_historical):
+            result = self.client.get_historical_data(
+                dataset="GLBX.MDP3",
+                symbols="MESU5",
+                schema="ohlcv-1m",
+                start="2025-01-01",
+                end="2025-01-02"
+            )
+            
+            # Should eventually succeed
+            self.assertIsInstance(result, pd.DataFrame)
+            self.assertEqual(len(result), 2)
+            
+            # Should have tried 3 times total
+            self.assertEqual(mock_historical.return_value.timeseries.get_range.call_count, 3)
+
+    def test_client_property_lazy_initialization(self):
+        """Test that client property properly handles lazy initialization"""
+        # Create new client instance
+        client = DataBentoClient("test_key")
+        
+        # Should start with None
+        self.assertIsNone(client._client)
+        
+        # Access should initialize 
+        with patch('lumibot.tools.databento_helper.Historical') as mock_historical:
+            _ = client.client
+            mock_historical.assert_called_once_with(key="test_key")
+        
+        # Should now be set
+        self.assertIsNotNone(client._client)
+
+    @patch('lumibot.tools.databento_helper.Historical')
+    def test_auth_retry_with_different_error_formats(self, mock_historical):
+        """Test authentication retry with various error message formats"""
+        test_cases = [
+            "401 auth_authentication_failed",
+            "HTTP 401: Authentication failed",
+            "401: Unauthorized access", 
+            "Authentication failed.",
+            "Invalid token",
+            "Token expired"
+        ]
+        
+        for error_msg in test_cases:
+            with self.subTest(error=error_msg):
+                mock_historical = Mock()
+                mock_historical.return_value.timeseries.get_range.side_effect = [
+                    Exception(error_msg),
+                    Mock(to_df=Mock(return_value=pd.DataFrame()))
+                ]
+                
+                client = DataBentoClient("test_key")
+                
+                with patch('lumibot.tools.databento_helper.Historical', mock_historical):
+                    try:
+                        result = client.get_historical_data(
+                            dataset="GLBX.MDP3",
+                            symbols="TEST",
+                            schema="ohlcv-1m",
+                            start="2025-01-01",
+                            end="2025-01-02"
+                        )
+                        # If auth-related error, should retry and succeed
+                        if any(keyword in error_msg.lower() for keyword in ['401', 'auth', 'authentication', 'unauthorized', 'token']):
+                            self.assertIsInstance(result, pd.DataFrame)
+                            self.assertEqual(mock_historical.call_count, 2)  # Original + retry
+                    except Exception:
+                        # Non-auth errors should fail immediately
+                        if not any(keyword in error_msg.lower() for keyword in ['401', 'auth', 'authentication', 'unauthorized', 'token']):
+                            self.assertEqual(mock_historical.call_count, 1)  # No retry
+
+    @patch('lumibot.tools.databento_helper.logger')
+    @patch('lumibot.tools.databento_helper.Historical')
+    def test_auth_retry_logging(self, mock_historical, mock_logger):
+        """Test that authentication retry attempts are properly logged"""
+        mock_historical.return_value.timeseries.get_range.side_effect = [
+            Exception("401 auth_authentication_failed"),
+            Mock(to_df=Mock(return_value=pd.DataFrame()))
+        ]
+        
+        with patch('lumibot.tools.databento_helper.Historical', mock_historical):
+            self.client.get_historical_data(
+                dataset="GLBX.MDP3", 
+                symbols="MESU5",
+                schema="ohlcv-1m",
+                start="2025-01-01",
+                end="2025-01-02"
+            )
+            
+            # Should log retry attempt
+            mock_logger.warning.assert_called_once()
+            warning_call = mock_logger.warning.call_args[0][0]
+            self.assertIn("authentication error", warning_call.lower())
+            
+            # Should also log client recreation
+            mock_logger.info.assert_called()
+            info_calls = [call[0][0] for call in mock_logger.info.call_args_list]
+            self.assertTrue(any("recreating" in call.lower() for call in info_calls))
